<x-mailform-form>
  <form onsubmit="{form_submit}">
    <div>
      <div each="{opts.fields}">
        <label for="{id}">{label}<span if="{required}">（必須）</span></label>
        <input type="{type}" ref="{name}" value="{value}" id="{id}" required="{required}" if="{isPlain}">
        <textarea ref="{name}" id="{id}" required="{required}" if="{isTextarea}">{value}</textarea>
        <ul>
          <li each="{i in selection}" if="{isRadio || isCheckbox}">
            <input ref="{name}" type="{type}" name="{name}" value="{i}" id="{id}-{i}" checked="{value.includes(i)}">
            <labelfor="{id}-{i}">{i}</label>
          </li>
        </ul>
        <select ref="{name}" name="{name}" multiple="{multiple}" required="{required}" if="{isOption}">
          <option each="{i in selection}" selected="{value.includes(i)}">{i}</option>
        </select>
      </div>
    </div>
    <div><input type="submit" value="{opts.useConfirm ? '確認' : '送信'}"></div>
  </form>

form_submit = function(event) {
  event.preventDefault();
  for (var _name in opts.fields) {
    var field = opts.fields[_name];
    var ref = this.refs[field.name];
    if (ref)
      field.value = field.isSelective ?
        mailform.selectedValue(ref) : ref.value;
  }
  if (opts.useConfirm)
    mailform.mount_action(mailform.action_confirm);
  else
    mailform.post();
};
</x-mailform-form>

<x-mailform-confirm>
  <div style="border:1px dotted #cccccc; background:#efefef;" class="pt-15 pl-15 pr-15 mb-20">
    <div each="{opts.fields}">
      <label class="underline">{label}<span if="{required}">（必須）</span></label>
      <p if="{!isTextarea}">{value}</p>
      <p if="{isTextarea}">{value}</p>
    </div>
  </div>
  <form onsubmit="{confirm_submit}">
    <input type="button" value="戻る" onclick="{confirm_return}">
    <input type="submit" value="送信">
  </form>

confirm_return = function(event) {
  event.preventDefault();
  mailform.mount_action(mailform.action_form);
};
confirm_submit = function(event) {
  event.preventDefault();
  mailform.post();
};
</x-mailform-confirm>

<x-mailform-post></x-mailform-post>

<x-mailform>
  <header />
  <main />
  <footer />
mailform.setup(opts);
this.on("mount", function() {
  mailform.mount_action(mailform.action_form);
});
</x-mailform>

var mailform = new (function() {
  this.action_form = "form";
  this.action_confirm = "confirm";
  this.action_post = "post";
  this.mount_action = function(action) {
    riot.mount("x-mailform main", "x-mailform-" + action, this.opts);
    riot.mount("x-mailform header", "x-mailform-header-" + action, this.opts);
    riot.mount("x-mailform footer", "x-mailform-footer-" + action, this.opts);
  };

  this.selectedValue = function(ref) {
    var r = [];
    for (var _name in ref) {
      var i = ref[_name];
      if (i.checked || i.selected)
        r.push(i.value);
    }
    return r;
  };

  this.postJSON = function() {
    var h = {};
    var undefined;
    for (var _name in this.opts.fields) {
      var field = this.opts.fields[_name];
      if (field.name !== undefined && field.value !== undefined)
        h[field.name] = field.value;
    }
    return JSON.stringify(h);
  };
  this.post = function() {
    fetch(this.opts.postTo, {
      method: "POST",
      body: this.postJSON(),
      credentials: "include",
    }).then(function(response) {
      if (!response.ok)
        throw new Error(response.status + " " + response.statusText);
      return response;
    }).then(function(response) {
      return response.text();
    }).then(function(text) {
      try {
        mailform.mount_action(mailform.action_post);
      } catch (e) {
      }
    }).catch(function(err) {
      alert("エラーが発生しました。\n" + err.message);
    });
  };

  this.setup = function(opts) {
    var undefined;
    this.opts = opts;
    this.opts.fields = (function(fields) {
      if (opts.fields) {
        var Field = function(obj) {
          var undefined;
          for (var i in obj)
            this[i] = obj[i];
          if (this.type === undefined)
            this.type = "text";
          if (this.label === undefined)
            this.label = this.name;
          this.id = "x-mailform-" + this.name;
          this.isTextarea = this.type == "textarea";
          this.isOption = this.type == "option";
          this.isCheckbox = this.type == "checkbox";
          this.isRadio = this.type == "radio";
          this.isSelective = this.isOption || this.isCheckbox || this.isRadio;
          this.isPlain = !(this.isTextarea || this.isSelective);
        };
        return fields.map(function(i) {return new Field(i);});
      } else
        return [];
    })(opts.fields);
    if (this.opts.useConfirm === undefined)
       this.opts.useConfirm = false;

    this.mount_action(this.action_form);
  };
})();
